
package jartege ; 

import java.lang.reflect.* ; 
import java.util.* ; 
import java.io.* ; 
import org.jmlspecs.jmlrac.runtime.* ; 

/**
 * Classe des classes Java.
 */
class Classe extends ElementAPoids {

   // Le dictionnaire
   Dictionnaire leDictionnaire ; 

   // L'objet de type Class reprsentant cette classe
   Class theClass ; 

   // L'ensemble des objets ayant pour type cette classe
   List lesObjets ; 

   // L'ensemble des constructeurs publics de cette classe
   List lesConstructeurs ; 

   // L'ensemble des champs publics de cette classe
   List lesChamps ; 

   // L'ensemble des oprations publiques de cette classe
   List lesOperations ; 

   // La classe permettant de gnrer les paramtres
   Classe classeMGP ; 

   // Les super-classes de cette classe
   List lesSuperClasses ; 

   // Seuil de cration d'objets de cette classe
   // CreationProbability seuil ; 

   // Les valeurs initiales des champs statiques de cette classe
   Map valInitChampsStat ; 

   // Le gnrateur d'objets associ  cette classe
   ObjectGenerator objectGen ; 

  
   // Constructeur
   Classe(Class theClass, Dictionnaire leDictionnaire) {


      if (theClass.isArray()) {
         Class componentType = theClass.getComponentType() ; 
         Classe typeComposant = leDictionnaire.getClasse(componentType) ; 
         setNomComplet(typeComposant.getNomComplet() + "[]") ; 
      } else if (theClass.getDeclaringClass() != null) {
// System.out.println("Classe : " + theClass.getName()) ; 
         Class declaringClass = theClass.getDeclaringClass() ; 
         Classe classeDec = leDictionnaire.getClasse(declaringClass) ; 
         String nomClasseDec = classeDec.getNomComplet() ; 
// System.out.println("nomClasseDec = " + nomClasseDec) ; 
         int indice = nomClasseDec.length() + 1 ; 
         String nomClasse = theClass.getName().substring(indice) ; 
// System.out.println("nomClasse = " + nomClasse + "\n") ; 
         setNomComplet(nomClasseDec + "." + nomClasse) ; 
      } else {
         setNomComplet(theClass.getName()) ; 
      }

      Trace.afficher(2, "Creation of class: " + getNomComplet()) ; 
      
      this.theClass = theClass ; 
      this.leDictionnaire = leDictionnaire ; 
      leDictionnaire.ajouter(this) ; 
      lesSuperClasses   = new Vector() ; 
      lesOperations     = new Vector() ; 
      lesConstructeurs  = new Vector() ; 
      valInitChampsStat = new HashMap() ; 
      initialiserValChampsStatiques() ; 

      // Construction du vecteur des objets
      lesObjets = new Vector() ; // A faire plus tot ?
      ajouterNul() ; 

      // seuil = ClassTester.creationProbabilityNormal ; 

      objectGen = new DefaultObjectGenerator(this) ; 

      String paq = Chaine.paquetage(getNomComplet()) ; 
      // String nomMGP = paq + "JRT_" + nom ; 
      String nomMGP = "JRT_" + nom ; 
      classeMGP = chercherClasse(nomMGP) ; 

      // Constructions du vecteur des super-classes et interfaces
      Class superC = theClass.getSuperclass() ; 
      Classe superClasse ; 
      if (superC != null) {
          superClasse = leDictionnaire.getClasse(theClass.getSuperclass()) ; 
          lesSuperClasses.add(superClasse) ; 
          lesSuperClasses.addAll(superClasse.lesSuperClasses) ; 
      }

      Class[] tabInterfaces = theClass.getInterfaces() ; 
      for (int i = 0 ; i < tabInterfaces.length ; i++) {
         Classe interf = leDictionnaire.getClasse(tabInterfaces[i]) ; 
         lesSuperClasses.add(interf) ; 
         lesSuperClasses.addAll(interf.lesSuperClasses) ; 
      }



      String packageName = leDictionnaire.classTester.getPackage() ; 
      Iterator it ; 
      Class toExtendClass = leDictionnaire.classTester.getToExtendClass() ; 

      // Construction du vecteur des oprations

      // ...  partir des mthodes
      Set tabMethodes = Constituant.getAllVisibleMethods(theClass, packageName, toExtendClass) ;  
      it = tabMethodes.iterator() ; 
      while (it.hasNext()) {
         Method method = (Method) it.next() ; 
         method.setAccessible(true) ; 
         String nomMethode = method.getName() ; 
         Class declaringClass = method.getDeclaringClass() ; 
         String nomClasse = declaringClass.getName() ; 

         if (! nomClasse.equals("java.lang.Object") && 
             ! Chaine.contientDollar(nomMethode)) {

            Trace.afficher(3, "Method: " + nomMethode) ; 

            Methode m = new Methode(method, this) ; 
            lesOperations.add(m) ; 

         }
      }

      // ...  partir des champs 
      Set tabField = Constituant.getAllVisibleFields(theClass, packageName, toExtendClass) ; 
      it = tabField.iterator() ;
      while (it.hasNext()) {
         Field theField = (Field) it.next() ; 
         String nomChamp = theField.getName() ; 
         theField.setAccessible(true) ; 

         if (! Chaine.contientDollar(nomChamp)) {
            Trace.afficher(3, "Field: " + nomChamp) ; 
            OperationGet opGet = new OperationGet(theField, this) ; 
            lesOperations.add(opGet) ; 
   
            int modifiers = theField.getModifiers() ; 

            if (!Modifier.isFinal(modifiers)) {
               OperationSet opSet = new OperationSet(theField, this) ; 
               lesOperations.add(opSet) ; 
            }
           
         }
      }

      // Construction du vecteur des constructeurs

      if (!Modifier.isAbstract(theClass.getModifiers())) {
         Set tabCons = Constituant.getAllVisibleConstructors(theClass, packageName, toExtendClass) ; 
         it = tabCons.iterator() ; 
         while (it.hasNext()) {
            Constructor theCons = (Constructor) it.next() ; 
            String nomCons = theCons.getName() ; 
            Trace.afficher(3, "Constructor: " + nomCons) ; 
            theCons.setAccessible(true) ; 
            Constructeur cons = new Constructeur(theCons, this) ; 
            lesConstructeurs.add(cons) ; 
            // On ajoute le constructeur dans chaque super-classe
            for (int j = 0 ; j < lesSuperClasses.size() ; j++) {
               Classe sup = (Classe) lesSuperClasses.get(j) ; 
               sup.lesConstructeurs.add(cons) ; 
            }
         }
      }

      
   }

   public Classe chercherClasse(String s) {
      Classe c = null ; 
      try {
         Class clas = Class.forName(s) ; 
         c = leDictionnaire.getClasse(clas) ;
      } catch (ClassNotFoundException except) {
         // Classe non trouve 
      } catch (LinkageError error) {
         // Classe trouve, mais on ne peut pas la charger
         System.err.println("Class " + s + " should be public.") ; 
         System.exit(1) ; 
      }
      return c ; 
   }

   // Ajoute l'objet obj  la liste des objets de cette classe, si 
   // la valeur est diffrente des objets dj stocks.
   // Ajoute galement l'objet obj dans la liste des objets des super-classes.
   public void ajouter(Valeur obj) {
      boolean trouve = false ; 
      if (theClass.isPrimitive()) {
         for (int i = 0 ; !trouve && i < lesObjets.size() ; i++) {
            Valeur var = (Valeur) lesObjets.get(i) ;
            trouve = (obj.valeur.equals(var.valeur)) ;
         }
      } else {
         for (int i = 0 ; !trouve && i < lesObjets.size() ; i++) {
            Valeur var = (Valeur) lesObjets.get(i) ; 
            trouve = (obj.valeur == var.valeur) ; 
         }
      }
      if (!trouve) {
         lesObjets.add(obj) ; 
         for (int i = 0 ; i < lesSuperClasses.size() ; i++) {
            Classe superClasse = (Classe) lesSuperClasses.get(i) ; 
            superClasse.ajouter(obj) ; 
         }
      }
   }

   public void ajouterNul() {
      if (!theClass.isPrimitive()) {
         Valeur valNulle = 
            new Valeur("(" + getNomComplet() + ") null", this, null) ; 
         ajouter(valNulle) ; 
         
      }
   }

   /**
    * Initialisation des champs statiques.
    */

   void initialiserValChampsStatiques() {
      Field[] champs = theClass.getDeclaredFields() ; 
      for (int i = 0 ; i < champs.length ; i++) {
         Field champ = champs[i] ; 
         int modifiers = champ.getModifiers() ; 
         if (!Chaine.contientDollar(champ.getName()) && 
             Modifier.isStatic(modifiers) && 
             !Modifier.isFinal(modifiers)) {
            champ.setAccessible(true) ; 
            Object val = null ; 
            try {
               val = champ.get(null) ; 
            } catch (IllegalAccessException except) {
               System.err.println("Erreur d'acces " + nom + " : " + champ.getName()) ; 
               System.exit(1) ; 
            }
            valInitChampsStat.put(champ, val) ; 
         }
      }
   } 

   /**
    * Reinitialisation des champs statiques.
    */

   void reinitialiserValChampsStatiques() {
      Collection champs = valInitChampsStat.keySet() ;
      Iterator it = champs.iterator() ; 
      while (it.hasNext()) {
         Field champ = (Field) it.next() ; 
         Object val = null ; 
         try {
            val = valInitChampsStat.get(champ) ; 
            champ.set(null, val) ; 
         } catch (IllegalAccessException except) {
            System.err.println("Erreur d'acces " + nom + " : " + champ.getName()) ;
            System.exit(1) ; 
         }
      }
   }


   // Generation d'un test de cette classe
   public void generer(FileWriter fichier) throws IOException {
      List lesOp = new Vector(lesOperations) ; 
      Expression resultat ; 
      boolean fini = false ; 
      while (lesOp.size() != 0 && !fini) {
         Operation op = (Operation) ElementAPoids.choisir(lesOp) ; 
         if (op != null) {
            Trace.afficher(7, "method: " + op.getNom()) ; 
            resultat = op.genererUnAppel(fichier) ; 
            fini = (resultat != null) ; 
            lesOp.remove(op) ; 
         } else {
            fini = true ; 
         }
      }
   }


   // Cr un nouvel objet de cette classe, si la classe comporte
   // peu d'objets.
   Expression genererUnObjet(boolean nonNul, FileWriter fichier) 
         throws IOException {
      return objectGen.generateAnObject(nonNul, fichier) ; 
   }

   boolean estPrimitif() {
      return theClass.isPrimitive() || nom.equals("String") ; 
   }

   Valeur choisirValeur(boolean nonNul) {
      Valeur resultat = null ; 
      int taille ; 
      int indice ; 
      if (nonNul) {
         taille = lesObjets.size() - 1 ;
         if (taille > 0) {
            indice = RandomValue.intValue(taille - 1) + 1 ;
            resultat = (Valeur) lesObjets.get(indice) ;
         } else {
            resultat = null ; 
            // On ne peut pas choisir un objet non null
         }
      } else {
         taille = lesObjets.size() ;
         indice = RandomValue.intValue(taille - 1) ;
         resultat = (Valeur) lesObjets.get(indice) ; 
      }
      return resultat ; 
   }


   // Generation d'un type de base
   Valeur genererValeurTypeBase() {
      Valeur valeur ; 
      if (nom.equals("int")) {
         int i = RandomValue.intValue() ; 
         Integer integ = new Integer(i) ; 
         valeur = new Valeur(this, integ) ; 
      } else if (nom.equals("long")) {
         long l = RandomValue.longValue() ; 
         Long lon = new Long(l) ; 
         valeur = new Valeur(this, lon) ; 
      } else if (nom.equals("float")) {
         float f = RandomValue.nonUniformFloatValue() ; 
         Float fl = new Float(f) ; 
         valeur = new Valeur(this, fl) ; 
      } else if (nom.equals("double")) {
         double d = RandomValue.nonUniformDoubleValue() ; 
         Double dou = new Double(d) ; 
         valeur = new Valeur(this, dou) ; 
      } else if (nom.equals("byte")) {
         byte b = RandomValue.byteValue() ; 
         Byte by = new Byte(b) ; 
         valeur = new Valeur(this, by) ; 
      } else if (nom.equals("short")) {
         short s = RandomValue.shortValue() ; 
         Short sh = new Short(s) ; 
         valeur = new Valeur(this, sh) ; 
      } else if (nom.equals("char")) {
         char c = RandomValue.charValue() ; 
         Character ch = new Character(c) ; 
         valeur = new Valeur(this, ch) ; 
      } else if (nom.equals("boolean")) {
         boolean b = RandomValue.booleanValue() ; 
         Boolean bo = new Boolean(b) ; 
         valeur = new Valeur(this, bo) ; 
      } else if (nom.equals("String")) {
         String s = RandomValue.stringValue() ; 
         valeur = new Valeur(this, s) ; 
      } else {
         valeur = new Valeur(this, null) ; 
      }
      return valeur ; 
   } 

}

