package jartege ; 

import java.util.* ; 

class Dictionnaire {

   // La table des classes
   Map classTable ; 

   // Le classTester auquel est associ ce dictionnaire
   ClassTester classTester ; 

   
   private final Valeur valeurVoid ;
   private final Valeur uneValInt ;
   private final Valeur uneValFloat ;
   private final Valeur uneValDouble ;
   private final Valeur uneValLong ;
   private final Valeur uneValBoolean ;
   private final Valeur uneValShort ;
   private final Valeur uneValByte ;
   private final Valeur uneValChar ;


   // Constructeur
   Dictionnaire(ClassTester classTester) {
      classTable = new Hashtable() ; 
      this.classTester = classTester ; 
      valeurVoid = new Valeur("ok", getClasse(Void.TYPE), null) ;
      uneValInt = new Valeur("0", getClasse(Integer.TYPE), new Integer(0)) ;
      uneValFloat = new Valeur("0.0F", getClasse(Float.TYPE), new Float(0)) ;
      uneValDouble = new Valeur("0.0", getClasse(Double.TYPE), new Double(0)) ;
      uneValLong = new Valeur("0L", getClasse(Long.TYPE), new Long(0)) ;
      uneValBoolean = 
         new Valeur("false", getClasse(Boolean.TYPE), new Boolean(false)) ;
      uneValShort = 
         new Valeur("0", getClasse(Short.TYPE), new Short((short) 0)) ;
      uneValByte = new Valeur("0", getClasse(Byte.TYPE), new Byte((byte) 0)) ;
      uneValChar = 
         new Valeur("'x'", getClasse(Character.TYPE), new Character('x')) ;

   }

   Valeur varVoid() {
      return valeurVoid ;
   }

   // Retourne l'objet de type Classe qui correspond  la classe spcifie
   Classe getClasse(String className) {
      Class theClass = Util.forName(className) ; 
      return getClasse(theClass) ; 
   }

   // Retourne l'objet de type Classe qui correspond  la Class spcifie
   Classe getClasse(Class theClass) {
      Classe classe = (Classe) classTable.get(theClass) ;
      if (classe == null) {
         classe = new Classe(theClass, this) ;
      }
      return classe ;
   }


   // Ajoute la classe spcifie dans le dictionnaire
   void ajouter(Classe laClasse) {
      classTable.put(laClasse.theClass, laClasse) ; 
   }

   // Retourne une valeur de la classe
   Valeur uneValeur(Classe classe) {
      if (classe.getNom().equals("int")) {
         return uneValInt ;
      } else if (classe.equals(getClasse(Float.TYPE))) {
         return uneValFloat ;
      } else if (classe.equals(getClasse(Double.TYPE))) {
         return uneValDouble ;
      } else if (classe.equals(getClasse(Long.TYPE))) {
         return uneValLong ;
      } else if (classe.equals(getClasse(Boolean.TYPE))) {
         return uneValBoolean ;
      } else if (classe.equals(getClasse(Byte.TYPE))) {
         return uneValByte ;
      } else if (classe.equals(getClasse(Short.TYPE))) {
         return uneValShort ;
      } else if (classe.equals(getClasse(Character.TYPE))) {
         return uneValChar ;
      } else {
         return new Valeur("null", classe, null) ;
      }
   }

   /**
    * Reinitialisation de toutes les classes.
    * Remets l'ensemble des objets de chaque classe  vide.
    * Reinitialise les champs statiques.
    */

   void reinit() {
      GenerateurNom.reinit() ;
      Collection listeClasses = classTable.values() ;
      Iterator it = listeClasses.iterator() ;
      while (it.hasNext()) {
         Classe classe = (Classe) it.next() ;
         classe.lesObjets = new Vector() ;
         classe.reinitialiserValChampsStatiques() ;
      }
   }

   /**
    * Supprime tous les objets de toutes les classes de ce dictionnaire.
    * Rinitialise le gnrateur de noms.
    */

   void viderObjets() {
      GenerateurNom.reinit() ;
      Collection listeClasses = classTable.values() ;
      Iterator it = listeClasses.iterator() ;
      while (it.hasNext()) {
         Classe classe = (Classe) it.next() ;
         classe.lesObjets = new Vector() ;
         classe.ajouterNul() ; 
      }
   }

   void compteRendu() {

      Collection listeClasses = classTester.lesClasses ;
      Iterator it = listeClasses.iterator() ;
      while (it.hasNext()) {
         Classe classe = (Classe) it.next() ;
/*
         System.out.println("Class: " + classe.getNomComplet()) ; 
*/
         Collection lesCons = classe.lesConstructeurs ; 
         Iterator itOp = lesCons.iterator() ;
         double sommePoids = 0 ; 
         int sommeAppels = 0 ; 
         while (itOp.hasNext()) {
            Constructeur cons = (Constructeur) itOp.next() ; 
            sommePoids = sommePoids + cons.getPoids() ; 
            sommeAppels = sommeAppels + cons.numberOfCalls ; 
         }
         itOp = lesCons.iterator() ;
         while (itOp.hasNext()) {
            Constructeur cons = (Constructeur) itOp.next() ;
            double poids = cons.getPoids() ; 
            double prob = 0 ; 
            if (sommePoids != 0) {
               prob = poids / sommePoids ; 
            }
            double prop = 0 ; 
            if (sommeAppels != 0) {
               prop = (double) cons.numberOfCalls / (double) sommeAppels ; 
            }
            if (prob != 0 && cons.numberOfCalls == 0) {
               System.out.println("Warning: no call of " + cons.toString() + 
                  " in class " + classe.getNomComplet()) ; 
            }
/*
            System.out.println(
               "   " + cons.toString() + ":") ; 
            System.out.println("      poids = " + aff2(poids) + 
               ", prob = " + aff2(prob) + 
               ", appels = " + cons.numberOfCalls + 
               ", prop = " + aff2(prop)) ; 
            cons.numberOfCalls = 0 ; 
*/
         }
         

         Collection lesOperations = classe.lesOperations ; 
         itOp = lesOperations.iterator() ;
         sommePoids = 0 ;
         sommeAppels = 0 ;
         while (itOp.hasNext()) {
            Operation op = (Operation) itOp.next() ;
            sommePoids = sommePoids + op.getPoids() ;
            sommeAppels = sommeAppels + op.numberOfCalls ;
         }
         itOp = lesOperations.iterator() ;
         while (itOp.hasNext()) {
            Operation op = (Operation) itOp.next() ;
            double poids = op.getPoids() ;
            double prob = 0 ; 
            if (sommePoids != 0) {
               prob = poids / sommePoids ;
            }
            double prop = 0 ; 
            if (sommeAppels != 0) {
               prop = (double) op.numberOfCalls / (double) sommeAppels ;
            }
            if (prob != 0 && op.numberOfCalls == 0) {
               System.out.println("Warning: no call of " + op.toString() + 
                  " in class " + classe.getNomComplet()) ; 
            }
/*
            System.out.println(
               "   " + op.toString() + ":") ; 
            System.out.println(
               "      poids = " + aff2(poids) +
               ", prob = " + aff2(prob) +
               ", appels = " + op.numberOfCalls +
               ", prop = " + aff2(prop)) ;
            op.numberOfCalls = 0 ; 
*/
         }
      }
/*
      System.out.println() ; 
*/

   }

   static String aff2(double d) {
      String res = Double.toString(d) + "00" ; 
      int index = res.lastIndexOf(".") ; 
      return res.substring(0, index + 3) ; 
   }

}



   








