package jartege ; 

/**
 * The class ThresholdProbability allows to create creation
 * probability functions defined by a <em>threshold value</em>.
 * A creation probablity function defines
 * the probability of creation of an object in function of the number
 * of already created objects. Threshold probability functions are defined 
 * by a <em>threshold value</em> which indicates when the probability 
 * goes from 1 to 0.<br>
 * <code> 
 * <ul>
 *    theFunction(nbCreatedObjects) = 1, 
 *   <ul>
 *       if nbCreatedObjects < getThreshold() ; <br>
 *   </ul>
 *    theFunction(nbCreatedObjects) = 0, 
 *   <ul>
 *       if nbCreatedObjects >= getThreshold().
 *   </ul>
 * </ul>
 * </code>
 *
 * @author Catherine Oriat
 */

public class ThresholdProbability implements CreationProbability {

   private final int threshold ; 

   /**
    * Constructs a new creation probability function defined by
    * the specified threshold.
    *
    * @param threshold the threshold of the creation probability function.
    */

   public ThresholdProbability(int threshold) {
      this.threshold = threshold ; 
   }

   /**
    * The creation probability function.
    *
    * @param nbCreatedObjects the number of already created objects.
    */

   public double theFunction(int nbCreatedObjects) {
      if (nbCreatedObjects < threshold) {
         return 1 ; 
      } else {
         return 0 ; 
      }
   }

   /**
    * Returns the threshold of this creation probability function.
    */

   public int getThreshold() {
      return threshold ; 
   }
}

